
#    MIDAS Access Client  for MessageLogger service

global env
source [file join $env(MIDASBASE) SOAP tcl MIDASAccessClient.tcl]    ;#   load default support procedures


global MessageLoggerServerHost MessageLoggerServerPort
global SOAPServerHost SOAPServerPort

if {[info exists MessageLoggerServerHost] == 0} then {
    if {[info exists SOAPServerHost] == 0} then {set MessageLoggerServerHost localhost} else {set MessageLoggerServerHost $SOAPServerHost}
}

if {[info exists MessageLoggerServerPort] == 0} then {
    if {[info exists SOAPServerPort] == 0} then {set MessageLoggerServerPort 8015} else {set MessageLoggerServerPort $SOAPServerPort}
}


variable uri "urn:MessageLoggerServer"
variable proxy "http://$MessageLoggerServerHost:$MessageLoggerServerPort/MessageLoggerServer"

set MessageLoggerClient__TimeOut 4000

 global tcl_platform
    switch $tcl_platform(os) {
       "Windows NT" {
           package require registry
           global env
           set env(ComputerName) [registry get HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\ComputerName\\ActiveComputerName ComputerName]
       }
       SunOS  -
       Linux  {
           global env
           set env(ComputerName) [exec uname -n]
       }
    }

#    create and configure the SOAP method

SOAP::create MessageLoggerClient__DefineMessage -proxy $proxy -uri $uri -timeout $MessageLoggerClient__TimeOut -version 1.2 \
     -name DefineMessage \
     -params {class string level string source string message string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create MessageLoggerClient__GetLastError -proxy $proxy -uri $uri -timeout $MessageLoggerClient__TimeOut \
     -name GetLastError \
     -params {} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request 

SOAP::create MessageLoggerClient__GetInformation -proxy $proxy -uri $uri -timeout $MessageLoggerClient__TimeOut \
     -name GetInformation \
     -params {} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request

SOAP::create MessageLoggerClient__ServerVerbose -proxy $proxy -uri $uri -timeout 1000 \
     -name ServerVerbose \
     -params {opt int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 

SOAP::create MessageLoggerClient__ServerImplementation -proxy $proxy -uri $uri -timeout 1000 \
     -name ServerImplementation \
     -params {opt int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply 



#    define a Tcl procedure which has some (optional) arguments

#    format of Tcl procedure call to define a message is:-        DefineMessage [-c class] [-l level] [-s source] message

proc DefineMessage  {args} {
    global env
    
    set Class "unknown"       ;# define some default values
    set Level "INFORMATION"

    set Source $env(ComputerName)
    set Message ""

    set l [llength $args]
    if {$l < 1}  {return 0}      ;# no message
    if {$l == 1} {
        set Message "$args"
    } else {
        for {set i 0} {$i < $l} {incr i} {
            switch -- [lindex $args $i] {
                -c   {incr i; set Class [lindex $args $i]}
                -l   {incr i; set Level [lindex $args $i]}
                -s   {incr i; set Source [lindex $args $i]}
                default {set Message [lindex $args [expr [llength $args]-1]]}
            }
            if {$Message != ""} {break}
        }
    }

   set z [catch {MessageLoggerClient__DefineMessage $Class $Level "$Source" "$Message"} m]  ;# use SOAP method to define the message
   if {$z != 0} {puts stderr "Error in DefineMessage: $m\n"}
}

# puts "DefineMessage defined with server at $proxy"

package provide MessageLoggerClient 1.0
