

# init.tcl---startup for eg-session, runs in root interp

set ThisRelease 030709

set TestFlag 0

set preferences(release.info.$ThisRelease) 0


set defaultINITlibs ""               ;#    Tasks created via INIT will always loads these libraries from lib_$OS
set defaultBFlibs   "eg"            ;#    Tasks created via BF will always loads these libraries from lib_$OS

proc do_source {f} {
   if {![file exists $f]} then {return}
   set z [catch {uplevel #0 source $f} m]
   if {$z != 0} then {puts stdout "in $f $m"}
}


proc exec-defaults {} {
   global env

   do_source [midas-convert-filename [file join $env(TCLLOCAL) defaults.tcl]]
   do_source [midas-convert-filename [file join $env(HOME)     .midas-defaults.tcl]]
   do_source [midas-convert-filename [file join $env(TCLLOCAL) eg-defaults.tcl]]
   do_source [midas-convert-filename [file join $env(HOME)     .eg-defaults.tcl]]
}

proc exec-environment {} {
   global env

   do_source [midas-convert-filename [file join $env(TCLLOCAL) environment.tcl]]
   do_source [midas-convert-filename [file join $env(HOME)     .midas-environment.tcl]]
}

proc exec-colour-defaults {} {
  global env
#
#    for tk load user colour selection
#

 if {$env(TCLWM) != "tk"} then {return}

   loadsource tkcolourscheme
   do_source [midas-convert-filename [file join $env(TCLLOCAL) tkcolourscheme.tcl]]
   do_source [midas-convert-filename [file join $env(HOME)     .midas-tkcolourscheme.tcl]]
   do_source [midas-convert-filename [file join $env(TCLLOCAL) eg-tkcolours.tcl]]
   do_source [midas-convert-filename [file join $env(HOME)     .eg-tkcolours.tcl]]
}

proc exec-font-defaults {} {
  global env
#
#    for tk load user font selection
#

 if {$env(TCLWM) != "tk"} then {return}

   loadsource fontscheme
   do_source [midas-convert-filename [file join $env(TCLLOCAL) fontscheme.tcl]]
   do_source [midas-convert-filename [file join $env(HOME)     .midas-fontscheme.tcl]]
   do_source [midas-convert-filename [file join $env(TCLLOCAL) eg-tkfonts.tcl]]
   do_source [midas-convert-filename [file join $env(HOME)     .eg-tkfonts.tcl]]
}

proc set-defaults {} {
   global env

 if {$env(TCLWM) == "tk"} then {

   exec-colour-defaults

#    if tkspectrumspeccolour exists modify inbuilt 1d spectrum colour map
#    note that system and/or user default may override this latter
   if {[info exists env(tkspectrumspeccolour)] == 1} then {
      if {$env(USERCOLORS) != 0} then {
         set map [get-eg-option linecolours]
         set map [lreplace $map 0 0 $env(tkspectrumspeccolour)]
         set-eg-option linecolours $map
      }
   }

   exec-font-defaults
   
  }

   exec-defaults
}

proc findautoloadfile {} {
  global env tcl_platform

    if {$tcl_platform(platform) == "unix"} then {set J ":"} else {set J ";"}
    set path [split $env(TCLPATH) $J]

    foreach dir $path {
       set file [file join $dir auto-load]
       if {[file exists [set file].tcl]} {
          set env(AUTOLOAD)   $file
          puts stdout "Autoload uses $env(AUTOLOAD).tcl"
          return
       }
    }
  puts stderr "can't find auto-load.tcl from $env(TCLPATH)"
}

proc findpackageloadfile {} {
  global env tcl_platform

    if {$tcl_platform(platform) == "unix"} then {set J ":"} else {set J ";"}
    set path [split $env(TCLPATH) $J]

    foreach dir $path {
       set file [file join $dir package-load]
       if {[file exists [set file].tcl]} {
          set env(PACKAGELOAD)   $file
          puts stdout "Package loading uses $env(PACKAGELOAD).tcl"
          return
       }
    }
  puts stderr "can't find package-load.tcl from $env(TCLPATH)"
}



#    init task execution starts here so

   set __interp root
   set mytaskname root

    global tcl_platform env

puts stdout "System identified is CPU $tcl_platform(machine); Platform is $tcl_platform(platform); OS is $tcl_platform(os) and Version is $tcl_platform(osVersion)"

if {$tcl_platform(platform) == "windows"} {

#    We have problems with HOME for Windows because of embedded spaces so change default

   set env(HOME) "C:/"
}
   
   if {$tcl_platform(platform) == "windows"} then {

#    for windows the command line which started the session
#    is available in argv
#    look for any environment variables defined via the command line

      foreach arg $argv {
         set name [lindex [split $arg =] 0]
         set value [lindex [split $arg =] 1]
         if {[string range $name 0 0] == "-"} then \
           {set name [string range $name 1 end]}
         set env($name) $value
         puts stdout "setting session variable $name = $value"
      }
    }
    
#    set window manager type to tk if not supplied
    if { ![info exists env(TCLWM)] } {set env(TCLWM) tk}
    
    if {$env(TCLWM) == "tk"} then {

        load {} Tk
        wm withdraw .

        puts stdout "Display depth is [winfo depth .]"

        if {$tcl_platform(platform) != "windows"} then {

#    check display visuals available

           set visuals [winfo visualsavailable .]
           if {[lsearch $visuals "truecolor 24"] < 0} then {
               puts stdout "Information - visual truecolor 24 is not available, Image display quality will be reduced"
           }
        }
    }

#    set up operating system information

   set env(platform) $tcl_platform(platform)
   set env(os)       $tcl_platform(os)
   set env(machine)  $tcl_platform(machine)

if {$tcl_platform(platform) == "windows"} {set env(os) Win}

     if {[info exists env(MIDASUSE64)]} {

         switch -glob $tcl_platform(machine) {
            *686 -
            x86_64   {
               set env(machine) "x64_64"   ;#  set a generic type for 64 bit CPUs running 64 bit OS
               if {$env(os) == "Linux"} {set env(os) Linux64}
            }
            intel -
            *386 -
            x86_32   {
               set env(machine) "x386" 
            }
            default {
               puts stderr "\n*** WARNING - $tcl_platform(machine) is not a known MIDAS Linux 64-bit platform - ***\n"
            }
         }
     }  else  {
         switch -glob $tcl_platform(machine) {
            *686 -
            x86_64   {
               set env(machine) "x686"   ;#  set a generic type for 64 bit CPUs running 32 bit OS
            }
            intel -
            *386 -
            x86_32   {
               set env(machine) "x386" 
            }
            *4u -
            *4c -
            *4m -
            i86pc -
            ppc {
            }
            default {
               puts stderr "\n*** WARNING - $tcl_platform(machine) is not a known MIDAS Linux 32-bit platform - ***\n"
            }
         }
     }

   switch $env(platform) {
     windows  {
        set libext "_Win"
        set binext "_Win"
        if { ![info exists env(USER)] } then {
           if {[info exists env(USERNAME)] } then {
              set env(USER) $env(USERNAME)
              puts stdout "Setting environment variable USER to $env(USER)"
           }
        }
        if { ![info exists env(HOME)] } then {
           cd; set env(HOME) "[pwd]"
           puts stdout "Setting environment variable HOME to $env(HOME)"
        }
        if { ![info exists env(MIDASTempDir)] } then {
           set env(MIDASTempDir) "C:/Temp"
        }
        set env(ENDIAN) little
     }
     unix   {
        if { ![info exists env(USER)] } then {
           if {[info exists env(LOGNAME)] } then {set env(USER) $env(LOGNAME)}
        }
        if { ![info exists env(MIDASTempDir)] } then {set env(MIDASTempDir) "/tmp"}
        switch -glob $env(os) {
           SunOS    {
              switch -glob $env(machine) {
                 *4u -
                 *4c -
                 *4m {
                    set libext "_SunOS"
                    set binext "_SunOS"
                    set env(ENDIAN) big
                 }
                 i86pc {
                    set libext "_SunOSx86"
                    set binext "_SunOSx86"
                    set env(ENDIAN) little
                 }
                 default {puts stderr "$env(machine) not supported"}
              }
           }
           Linux  {
              switch $env(machine) {
                 x64_64 -
                 x686 -
                 x386 {
                    set libext "_Linux"
                    set binext "_Linux"
                    set env(ENDIAN) little
                 }
                 ppc {
                    if {![info exists env(processor)]} {set env(processor) ppc_4xx}
                    set libext "_Linux-$env(processor)"
                    set binext "_Linux-$env(processor)"
                    set env(ENDIAN) big
                 }
                 default {puts stderr "$env(machine) not supported"}
              }
           }
           Linux64  {
              switch $env(machine) {
                 x64_64 {
                    set libext "_Linux64"
                    set binext "_Linux64"
                    set env(ENDIAN) little
                 }
                 default {puts stderr "$env(machine) not supported"}
              }
           }
           default  {puts stderr "$env(os) not supported"}
        }
     }
     default {puts stderr "$env(platform) not supported"}
   }

puts stdout "Environment selected is CPU $env(machine); Platform $env(platform) and OS $env(os)"

set startupscript 0
if { [info exists env(TCLPATH)] } then {set startupscript 1}

#    set defaults for non defined environment variables

if {$env(platform) == "unix"} then {

#    next is being superceeded
   if { ![info exists env(EGHOME)] } {set env(EGHOME) /MIDAS}

   if { ![info exists env(MIDASBASE)] } {set env(MIDASBASE) /MIDAS}
} else {

#    next is being superceeded
   if { ![info exists env(EGHOME)] } {set env(EGHOME) [file dirname [pwd]]}

   if { ![info exists env(MIDASBASE)] } {set env(MIDASBASE) [file dirname [pwd]]}
}

if { ![info exists env(TCLHOME)] }  {set env(TCLHOME) [file join $env(MIDASBASE) tcl]}
if { ![info exists env(TCLLOCAL)] } {set env(TCLLOCAL) [file join $env(TCLHOME) local]}

#    look for a local environment file - load delayed until here so TCLLOCAL defined

#   load common.tcl
   source [file join $env(TCLHOME) common.tcl]

#   load user preferences

   load_preferences

   exec-environment

if {$env(platform) == "unix"} then {set J ":"} else {set J ";"}

if { ![info exists env(TCLPATH)] }  {
    set env(TCLPATH) $env(TCLHOME)
} else {
    set env(TCLPATH) $env(TCLPATH)$J[file join $env(MIDASBASE) tcl]
}

if { ![info exists env(MIDASEXPBASE)] } {set env(MIDASEXPBASE) [file join $env(MIDASBASE) experiments]}
if { ![info exists env(MIDASAPPLBASE)] } {set env(MIDASAPPLBASE) $env(MIDASBASE)}

if { ![info exists env(MIDASXMBASE)] } {set env(MIDASXMBASE) [file join $env(MIDASBASE) log]}

#    session security initialisation
if { ![info exists env(MIDASSECURITY)] } {set env(MIDASSECURITY) [file join $env(MIDASBASE) security]}
if { ![file exists $env(MIDASSECURITY)] || ![file isdirectory $env(MIDASSECURITY)]} {set env(MIDASSECURITY) "disabled"}

#if { ![info exists env(EXPERIMENTBASE)] } {set env(EXPERIMENTBASE) [file join $env(MIDASEXPBASE) NULL]}
#if { ![info exists env(EXPERIMENTDIR )] } {set env(EXPERIMENTDIR)  [file join $env(MIDASEXPBASE) NULL]}

if { ![info exists env(EXPUSEENDIAN)] } then {set env(EXPUSEENDIAN) "false"}

#    next is being superceeded
if { ![info exists env(EXPBASE)] } {set env(EXPBASE) $env(MIDASBASE)}

if { ![info exists env(TCLVERSION)] } then {set env(TCLVERSION) $tcl_version}

if { $env(TCLWM) == "tk" && ![info exists env(TKVERSION)] } then {set env(TKVERSION) $tk_version}

if { ![info exists env(MIDAS_LIBRARY)] }  {
   if {$env(platform) == "unix"} then {
#    for Unix library search path is different from program search path
      set env(MIDAS_LIBRARY) [file join $env(MIDASBASE) lib$libext]
   }
   if {$env(platform) == "windows"} then {
#    for Windows library search path is the same as program search path
      set env(MIDAS_LIBRARY) [file join $env(MIDASBASE) bin$libext]
   }
}

if {$env(os) == "SunOS" && $env(machine) == "sun4m"} then {
    set env(MIDAS_LIBRARY) [file join $env(MIDASBASE) lib_SunOS_4m]$J$env(MIDAS_LIBRARY)
}

if {$env(platform) == "unix"} then {
   if { ![info exists env(LD_LIBRARY_PATH)] } then {
      set env(LD_LIBRARY_PATH) $env(MIDAS_LIBRARY)
   } else {
      if {"$env(LD_LIBRARY_PATH)" != "$env(MIDAS_LIBRARY)"} then \
            {set env(LD_LIBRARY_PATH) $env(MIDAS_LIBRARY)$J$env(LD_LIBRARY_PATH)}
   }
   puts stdout "library search path is $env(LD_LIBRARY_PATH)"
   puts stdout "program search path is $env(PATH)"
}
if {$env(platform) == "windows"} then {
   set env(PATH) $env(MIDAS_LIBRARY)$J$env(PATH)
   puts stdout "program & library search path is $env(PATH)"
}

if { ![info exists env(TCL_LIBRARY)] } {set env(TCL_LIBRARY) [file join $env(MIDASBASE) lib tcl$env(TCLVERSION)]}
if {$env(TCLWM) == "tk" && ![info exists env(TK_LIBRARY)] }  {set env(TK_LIBRARY)  [file join $env(MIDASBASE) lib tk$env(TKVERSION)]}

#    set icon base for TK use
if {$env(TCLWM) == "tk" && ![info exists env(ICONHOME)] } {set env(ICONHOME) [file join $env(TCLHOME) bitmaps]}

#    the following is possible for Windows systems
if {![info exists env(USER)] || $env(USER) == "" } then {
    puts stdout "environment variable USER not defined - setting to \"unknown\""
    set env(USER) unknown
}

findautoloadfile

findpackageloadfile

#   load auto-load support
source [file join $env(TCLHOME) auto-load.tcl]


#    tasks created via root
#    these tasks do not require to be defined using a bf.include file

if {$env(TCLWM) == "tk"} then {

set tasklist {
  {rmtcntrl                % eg   #  TK remote monitor task}
  {rmt                            #  TK remote monitor task}
  {opt-misc                       #  miscellaneous option selection}
  {opt-colours             % eg   #  colour selection}
  {opt-fonts                      #  font selection}
  {opt-demos                      #  Tcl script demos and tests}
  {ftpclient                      #  Ftp Client}
  {htmlviewer                     #  HTML Viewer}
  {terminal                       #  terminal emulator}
  {bf                             #  base frame}
  {help                           #  online help}
  {sl                             #  system log}
  {pedit                          #  preference editor}
  {oedit                          #  experimental option editor}
  {editor                         #  text editor}
  {xm                      % eg   #  exception monitor}
  {ex                      % eg   #  experiment access}
  {ap                      % eg   #  apparatus access}
  {security                % eg   #  security checking}
  {sd                      % eg   #  spectrum display}
  {sb                      % eg   #  spectrum directory browser}
  {speccopy                % eg   #  spectrum save/copy}
  {db                      % eg   #  directory browser}
  {pdb                     % eg   #  polyarray directory browser}
  {opt                     % eg   #  spectrum options}
  {cmdb                    % eg   #  colourmap directory browser}
  {cal                     % eg   #  spectrum calibrate}
  {add                     % eg   #  spectrum add}
  {cut                     % eg   #  spectrum cuts}
  {idb                     % eg   #  spectrum import}
  {exp                     % eg   #  spectrum export}
  {str                     % eg   #  spectrum stretch}
}

} else {

set tasklist {
  {httpd                  #  Httpd}
}

}

proc create_ {task files libs} {
   global env TestFlag

#    see if security option enabled and check permissions if it is
   if {$task != "security" && $env(MIDASSECURITY) != "disabled"} then {
      if {![send_task security validate "$task,c"]} then {
#    user does not have permission to create this task
         set msg "Access Denied\n\nA password is required to create task $task and you have not supplied it"
         set z [catch {midas_dialog [self frame-name].warningDialog "Attempted Security Violation" \
           "$msg" "warning" 0 "OK" "supply password"} m]
         if {$z != 0} then {
            puts stderr "midas-warning: [self frame-name].warningDialog: $m"
            catch {destroy [self frame-name].warningDialog}
         } else {
            if {$m == 1} then {inform security start}
         }
         return "Access Denied"
      }
   }

   interp create $task
   puts stdout "Createtask: $task"
   if {$env(platform) == "unix"} then {
      if {[file exists $env(MIDASTempDir)] == 0} then {
         puts stderr "Temporary file directory $env(MIDASTempDir) does not exist\n\
               You can set the environment variable MIDASTempDir to define an alternative location"
      } else {
         catch {:file mkdir [file join $env(MIDASTempDir) tcl[pid]]}
         cd [file join $env(MIDASTempDir) tcl[pid]]
      }
   } else {
#    make library the current directory - this allows load of dynamic libraries
         cd $env(MIDAS_LIBRARY)
   }
   puts stdout "[pwd] is now current directory"
   interp eval $task "set __interp $task"
   if {$env(TCLWM) == "tk"} then {
       interp eval $task "load {} Tk"
       interp eval $task "wm withdraw ."
   }
   foreach lib $libs {
      switch $lib {
          eg       {set library [set lib]_[join [split [info tclversion] .] ""][info sharedlibextension]}
          default  {set library lib[set lib][info tclversion][info sharedlibextension]}
      }
      if {$TestFlag != 0} then {
         puts stdout "$task requested library $lib"
      } else {
         set z [catch {interp eval $task "load $library $lib"} m]
         if {$z != 0} then {
            puts stderr "$m"
  #          puts stdout "Critical error - session will now terminate"
  #          exit
         } else {
            puts stdout "$task loaded library $library  $lib"
         }
      }
   }

   interp eval $task "source $env(AUTOLOAD).tcl"
   interp eval $task "MIDASauto-load {common $files $task}"
   interp eval $task "source $env(PACKAGELOAD).tcl"
   interp eval $task "init-packages"
   if {$env(platform) == "unix" && $env(TCLWM) == "tk"} then {interp eval $task "rename send Xsend"}
   if {$task == "bf"} then {bf alias bf_tasks bf_tasks}
   $task alias send   send_task
   $task alias inform send_task
   $task alias delete_task delete_task
   $task alias create_task create_task
   $task alias Get_Task_Names Get_Task_Names
}

proc delete_task {task} {interp delete $task}

proc create_task {task} {task.$task}

proc Get_Task_Names {} {return [linsert [interp slaves] 0 root]}

#    local version of inform since root does not have the alias
proc inform {task action} {send_task $task $action}

set informmsglevel 0

proc send_task {task args} {
  global env informmsglevel

   if {$informmsglevel > 2} then {
     puts stdout "called send_task with $task ([llength $args])  $args"
   }

#    split up the args -

  set i 0
  while {$i < [llength $args]} {set arg($i) "[lindex $args $i]"; incr i}

  if {[lsearch [Get_Task_Names] $task] < 0} then {

#    task does not exist so create

        task.$task
  }

#    if args = null then was a create only request

  if {$args == "null"} then {return ""}

  if {$task != "root"} then {

#    pass on the message

     switch [llength $args] {
       1  {set z [catch {interp eval $task $arg(0)} m]}
       2  {set z [catch {interp eval $task $arg(0) $arg(1)} m]}
       3  {set z [catch {interp eval $task $arg(0) $arg(1) $arg(2)} m]}
       4  {set z [catch {interp eval $task $arg(0) $arg(1) $arg(2) $arg(3)} m]}
       5  {set z [catch {interp eval $task $arg(0) $arg(1) $arg(2) $arg(3) $arg(4)} m]}
       6  {set z [catch {interp eval $task $arg(0) $arg(1) $arg(2) $arg(3) $arg(4) $arg(5)} m]}
       default {puts stdout "inform called with too many args ([llength $args]) $task $args}
     }
  } else {

     switch [llength $args] {
       1  {set z [catch {eval $arg(0)} m]}
       2  {set z [catch {eval $arg(0) $arg(1)} m]}
       default {puts stdout "inform called with too many args ([llength $args]) $task $args}
     }
  }

  if {$z == 0 && $m != ""} then {if {$informmsglevel > 1} then {puts stdout "send_task in root: $task returned $z $m"}}
  if {$z != 0} then {if {$informmsglevel > 0} then {puts stderr "send_task in root: $task returned $z $m"}}
  if {$m != ""} then {if {$informmsglevel > 3} then {puts stdout "send_task in root: $m"}}
  if {$z != 0 && $arg(0) == "start"} then {if {$informmsglevel == 0} then {puts stderr "$task proc start returned $m"}}
  return "$m"
}


if {$env(TCLWM) == "tk"} then {
  
# original TK remote monitor task  -   see also rmtcntrl
proc ttask.rmt {} {
  global env
  interp create rmt
  puts stdout "Createtask: rmt"
  interp eval rmt {set __interp rmt; load {} Tk; source $env(AUTOLOAD).tcl; MIDASauto-load {rmt}}
   rmt alias Get_Task_Names Get_Task_Names
}
}

foreach taskitem $tasklist {

     set task [first $taskitem]
     set rest [lrange $taskitem 1 end]
     if {[string first % $rest] == -1} then {
        if {[first $rest] == "#"} {set files ""} else \
           {set files [string range $rest 0 [expr [string first # $rest]-1]]}
        set libs ""
     } else {
        set files [string range $rest 0 [expr [string first % $rest]-1]]
        set rest [string range $rest [expr [string first % $rest]+1]  end]
        set libs [string range $rest 0 [expr [string first # $rest]-1]]
     }

  set p "create_ $task \"$files\" \"$defaultINITlibs $libs\""
  proc task.$task {} $p
}

#    This creates ALL tasks started via bf
proc bf_tasks {} {
  global env defaultBFlibs
  set tasklist $env(bf_tasklist)

  foreach taskitem $tasklist {
     set task [first $taskitem]
     set rest [lrange $taskitem 1 end]
     if {[string first % $rest] == -1} then {
        if {[first $rest] == "#"} {set files ""} else \
           {set files [string range $rest 0 [expr [string first # $rest]-1]]}
        set libs ""
     } else {
        set files [string range $rest 0 [expr [string first % $rest]-1]]
        set rest [string range $rest [expr [string first % $rest]+1]  end]
        set libs [string range $rest 0 [expr [string first # $rest]-1]]
     }

     set p "create_ $task \"$files\" \"$defaultBFlibs $libs\"" 
     proc task.$task {} $p
  }
}

proc setup-printers {} {
   global env

   switch $env(platform)  {
      windows {
         switch -glob $env(os) {
            "*NT" {return [setup-printers-windows]}
            "*9*" {return [setup-printers-windows]}
            default {}
         }
      }
      unix  {
         switch $env(os) {
            SunOS  {return [setup-printers-SunOS]}
            Linux  {return [setup-printers-Linux]}
            OSF1   {return [setup-printers-OSF1]}
            default {}
         }
      }
      default {}
   }
   return "none"
}

proc setup-printers-windows {} {
#    as a proc so that it can be patched

puts stdout "Using lplist to configure printers"

   set z [catch {exec lplist} m]
   if {$z == 0} then {
      set p [split $m \n]
      set names {}
      foreach entry $p {
        lappend names $entry
      }
   } else {
      puts stderr "$m"
      set names "none"
   }
   return $names
}


proc setup-printers-SunOS {} {
#    as a proc so that it can be patched

puts stdout "Using lpstat -v to configure printers"

   set z [catch {exec lpstat -v} m]
   if {$z == 0} then {
      set p [split $m \n]
      set names {}
      foreach entry $p {
        set printer [first [split [third $entry] :]]
        if {$printer == "_default"} then {set printer [default-printer-SunOS]}
        lappend names $printer
      }
   } else {
      puts stderr "$m"
      set names "none"
   }
   return $names
}

proc setup-printers-OSF1 {} {
#    as a proc so that it can be patched

puts stdout "Using lpstat -v to configure printers"

   set z [catch {exec lpstat -v} m]
   if {$z == 0} then {
      set p [split $m \n]
      set names {}
      foreach entry $p {
        lappend names [fourth $entry]
      }
   } else {
      puts stderr "$m"
      set names "none"
   }
   return $names
}

proc setup-printers-Linux {} {
#    as a proc so that it can be patched

puts stdout "Using lpc status to configure printers"

   set z [catch {exec /usr/sbin/lpc status} m]
   if {$z == 0} then {
      set p [split $m \n]
      set names {}
      foreach entry $p {
         set entry [string trim $entry]
         set name [first [split $entry :]]
         if {$name != $entry} then {lappend names $name}
      }
   } else {
      puts stderr "$m"
      set names "none"
   }
   return $names
}


proc default-printer {} {
   global env

   switch $env(platform)  {
      windows {
         switch -glob $env(os) {
            "*NT" {return [default-printer-windows]}
            "*9*" {return [default-printer-windows]}
            default {}
         }
      }
      unix  {
         switch -glob $env(os) {
            OSF1 -
            SunOS  {return [default-printer-SunOS]}
            Linux  {
#    BSD lpr assumes default printer has the name lp
               return "lp"
            }
            default {}
         }
      }
      default {}
   }
   return "none"
}

proc default-printer-windows {} {

   puts stdout "Using lplist -d to configure default printer"

   set z [catch {exec lplist -d} m]
   if {$z == 0} then {
      set name $m
   } else {
      puts stderr "$m"
      set name "none"
   }
   return $name
}

proc default-printer-SunOS {} {

   puts stdout "Using lpstat -d to configure default printer"

   set z [catch {exec lpstat -d} m]
   if {$z == 0} then {
      set name [string trim [second [split [first [split $m \n]] :]]]
   } else {
      puts stderr "$m"
      set name "none"
   }
   return $name
}

proc restart-bf {} {after 2000 inform bf start}
proc restart-sl {} {global env; if {$env(TCLWM) != "tk"} then {return}; after 2000 inform sl start}
proc restart-help {} {global env; if {$env(TCLWM) != "tk"} then {return}; after 2000 inform help start}

# establish some system defaults in case local or user
# default files are missing

set env(EGOPTIONS) { }

if {$env(TCLWM) == "tk"} then {

   source [file join $env(TCLHOME) config-tk.tcl]

#    setup colour options
if {[info exists env(PALETTE)]} then {set option $env(PALETTE)} else {set option "default"}
colour-config $option

#    now the very much harder task of fonts

   switch $env(platform)  {
      windows {
         if {[info exists env(FONTFMLY)]}  then {set fmly $env(FONTFMLY)} else {set fmly "clean"}
         if {[info exists env(FONTSIZE)]}  then {set size $env(FONTSIZE)} else {set size "medium"}
         font-config_8x  "$fmly" $size
      }
      unix  {
         switch  $env(os) {
            SunOS  {
               switch  [exec uname -r] {
                  5.5.1  -
                  5.5  {
                    if {[info exists env(FONTFMLY)]}  then {set fmly  $env(FONTFMLY)}   else {set fmly "times"}
                    if {[info exists env(FONTSIZE)]}  then {set size  $env(FONTSIZE)}   else {set size "medium"}
                    font-config "$fmly" $size
                  }
                  default {
                    if {[info exists env(FONTFMLY)]}  then {set fmly  $env(FONTFMLY)}   else {set fmly "times"}
                    if {[info exists env(FONTSIZE)]}  then {set size  $env(FONTSIZE)}   else {set size "medium"}
                    font-config_8x  "$fmly" $size
                  }
               }
            }
            Linux  -
            OSF1   {
               if {[info exists env(FONTFMLY)]}  then {set fmly  $env(FONTFMLY)}   else {set fmly "times"}
               if {[info exists env(FONTSIZE)]}  then {set size  $env(FONTSIZE)}   else {set size "medium"}
               font-config_8x  "$fmly" $size
            }
            default {}
         }
      }
      default {}
   }
   
}    ;#   end of       if {$env(TCLWM) == "tk"}

set p [setup-printers]

set-eg-option printer [default-printer]

set printernames {}
foreach entry $p {
   lappend printernames [list $entry $entry]
   puts stdout "adding printer $entry"
}
lappend printernames "file file"
puts stdout "adding printer file"

set-eg-option printernames $printernames

switch $env(platform)  {
   windows {
      switch -glob $env(os) {
         "*NT" {set-eg-option printtemplate "lpwin -d \"%printer\""}
         "*9*" {set-eg-option printtemplate "lpwin -d \"%printer\""}
         default {}
      }
   }
   unix  {
      switch $env(os) {
         OSF1 -
         SunOS  {set-eg-option printtemplate "lp -d %printer"}
         Linux  {set-eg-option printtemplate "lpr -P %printer"}
         default {}
      }
   }
   default {}
}
puts stdout "Using print command [get-eg-option printtemplate]"

switch $env(platform)  {
   windows {set-eg-option nulldevice "nul:"}
   unix    {set-eg-option nulldevice "/dev/null"}
   default {}
}

set-eg-option printtofiledir "$env(HOME)"

if {$env(TCLWM) == "tk"} {

set-eg-option default-browser-resource ""
set-eg-option default-browser-directory ""
set-eg-option spectrum-movement 1

set-eg-option backpeak {{backtype flatline} {peaktype gauss} {weight count} \
   {shape differen} {shapesfile no} {peaks 1} {order 1}}

set-eg-option brumfit {{shape gauss} {widthopt different} {peakopt different} \
   {order 2} {widths {5.0}} {skews {10.0}}}

set-eg-option peakfind {{low-chan 0} {high-chan 4095} {fwhm 5} {accept 5}}

set-eg-option gainmatch {{theta 0.0} {velocity 0.0} {xrayfilter 50} {gain 1.0} \
   {wcal0 5.0} {wcal1 0.0} {polyorder 2} {tolerance 4.0} {config NULL} {options 0}}

set-eg-option autocalibrate {{low-chan 1} {high-chan 4095} {Eu-accept 20} {Eu-minpeaks 7} \
   {Eu-maxpeaks 11} {Co-minpeaks 2} {Co-maxpeaks 2}}

set-eg-option specplot {{low-chan 0} {high-chan 4095} {strips 2} {norm linear} \
   {scale 200} {peaks 0} {gains {{0 4095 0 16384}}} {paper A4}}

set-eg-option exception-monitor 0

}     ;#    end of  if {$env(TCLWM) == "tk"}

set-defaults

puts stdout "Currently selected printer is [get-eg-option printer]"

#    reset printers to remove obselete user option set
set-eg-option printernames $printernames

if {$env(TCLWM) == "tk"} then {

#   load common.tk.tcl to obtain TK support
    source [file join $env(TCLHOME) common.tk.tcl]
}

#   load application data
if {[file exists [file join $env(TCLHOME) init-applications.tcl]]} then \
   {source [file join $env(TCLHOME) init-applications.tcl]}
   
if {$env(TCLWM) == "tk"} then {

set frame INIT
set frametitle "Midas Application Selection"

proc request-application {} {
  global env applications

  :frame [self frame-name]   :label "[self frame-title]"  :toplevel T :colours H
  :panel P :layout H :padx 10 :pady 10

  :col-gap 0

  if {[winfo depth [self frame-name]] > 8} then {
     image create photo midas -format gif  -file [file join $env(ICONHOME) smallmidas.gif]
  } else {
     image create photo midas -format gif -palette 3/3/3 -file [file join $env(ICONHOME) smallmidas.gif]
  }

  :canvas MIDASICON :w 64 :h 64
  [:path MIDASICON] create image 36 36  -image midas

  :col-gap 20
  :choice-stack APPLICATION :label "Select Application you require:" :strings $applications :value 0

  :show T
}

proc release_info {ID} {
   global env

#    read the file

   set file [file join $env(MIDASBASE) release $ID.txt]
   if {![file exists $file]} then {return 0}

   set z [catch {open $file} f]
   if {$z != 0} then {midas-warning $f; return 0}
   set text [read $f]
   close $f

  :frame ".rlseinfo" :label "Information for MIDAS release ID $ID" \
       :show-footer F  :resize F :toplevel T :transient T

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout V; :row-gap 5; :col-gap 10
      :galley GALLEY  :scroll V :rows 15 :cols 80 :deaf T
      [:path GALLEY].text configure -background $env(tkframebackground)
      :modify GALLEY  :createtag "title -foreground red -background $env(tkframebackground)"
      :modify GALLEY  :createtag "body  -foreground black -background $env(tkframebackground)"
      set ttext [first [split $text \n]]
      :modify GALLEY  :usingtag title :strings "$ttext\n"
      :modify GALLEY  :usingtag body  :strings  "[string range $text [string length $ttext] end]"

      :modify GALLEY :see 0

      :next-row T

     :layout H
     :button BUTTON :label "Dismiss"                :command "destroy-frame .rlseinfo"
     :button RLSE   :label "Do not show this again" :command "suppress_release_info"

   :show T

   return 1
}

proc suppress_release_info {} {
    global preferences ThisRelease

    set preferences(release.info.$ThisRelease) 1
}

proc click-quit args {exit}

proc click-application {w} {
  global applications

  wm withdraw [self frame-name]
  start-application [lindex $applications $w]
}

}    ;#  end of   if {$env(TCLWM) == "tk"}

proc start-application {appl} {
   global env
   
if {$env(TCLWM) == "tk"} then {

   puts stdout "Using Tcl $env(TCLVERSION) & Tk $env(TKVERSION)"
   puts stdout "Using Tcl library $env(TCL_LIBRARY) and Tk library $env(TK_LIBRARY)"
} else {

   puts stdout "Using Tcl $env(TCLVERSION)"
   puts stdout "Using Tcl library $env(TCL_LIBRARY)"
}

   if {$appl != ""} then {
      puts stdout "Starting Application $appl"
      select-application $appl
   }

   puts stdout "Using dynamic library $env(MIDAS_LIBRARY)"

   puts stdout "Using TCL path $env(TCLPATH)"

   loadsource application-environment

#   if {$env(MIDASUSEORBIX) == "true"} then {puts stdout "Including Orbix communications"}

   if {$env(TCLWM) != "tk"} then {set env(MIDASSECURITY) "disabled"}
   if {$env(MIDASSECURITY) != "disabled"} then {inform security initialise} else {puts stdout "Security option NOT enabled"}

   loadsource init.patch
   start-session
}

#    provided so that a patch file can provide custom session startup options
proc start-session {} {
  global env
  if {$env(TCLWM) == "tk"} then {
     if {[info exists env(MIDASTASK)]} then {inform bf "start $env(MIDASTASK)"} else {inform bf start}
  } else {
     if {[info exists env(MIDASTASK)]} then {inform $env(MIDASTASK) start}
  }
}

if {$preferences(release.info.$ThisRelease) == 0} then {
    if {$env(TCLWM) == "tk"} then {release_info $ThisRelease}
}

if {[info exists env(MIDASAPPLICATION)]} then {
   start-application $env(MIDASAPPLICATION)
} else {
   if {$startupscript != 0} then {
       start-application ""
   } else {
       if {$env(TCLWM) == "tk"} then {request-application} else {start-application ""}
   }
}

if {[info exists env(MIDASTASK)]} {if {$env(MIDASTASK) == "httpd"} {vwait forever}}

#inform rmt start
#inform rmtcntrl start
